searchData={"items":[{"type":"module","title":"ExNudge.Telemetry","doc":"Telemetry events emitted by ExNudge.","ref":"ExNudge.Telemetry.html"},{"type":"module","title":"Events - ExNudge.Telemetry","doc":"### `[:ex_nudge, :send_notification]`\nThis event is emitted when a notification has completed both successfully and with an error.\n\n#### Measurements\n* `:duration` - The time taken in milliseconds from sending the notification to receiving a response.\n* `:payload_size` - Size of the payload in bytes\n\n#### Metadata\n* `:endpoint` - Sanitized push service endpoint\n* `:status` - Operation status (`:success` or `:error`)\n* `:http_status_code` - HTTP response status code or nil\n* `:error_reason` - Error reason or nil","ref":"ExNudge.Telemetry.html#module-events"},{"type":"module","title":"Example Usage - ExNudge.Telemetry","doc":":telemetry.attach(\"my-handler\", [:ex_nudge, :send_notification], fn name, measurements, metadata, config ->\n      case metadata.status do\n        :success ->\n          Logger.info(\"Notification sent successfully\",\n            duration: measurements.duration,\n            endpoint: metadata.endpoint)\n        :error ->\n          Logger.error(\"Notification failed\",\n            error: metadata.error_reason,\n            http_status: metadata.http_status_code)\n      end\n    end, nil)","ref":"ExNudge.Telemetry.html#module-example-usage"},{"type":"module","title":"ExNudge","doc":"A pure Elixir library for sending Web Push notifications.\n\nThis library implements the Web Push Protocol as defined in RFC 8291\nwith support for VAPID authentication and payload encryption as per RFC 8292.","ref":"ExNudge.html"},{"type":"module","title":"Quick Start - ExNudge","doc":"# Configure VAPID keys in your config\n    config :ex_nudge,\n      vapid_subject: \"mailto:your-email@example.com\",\n      vapid_public_key: \"your_public_key\",\n      vapid_private_key: \"your_private_key\"\n\n    # Send a notification\n    subscription = %ExNudge.Subscription{\n      endpoint: \"https://fcm.googleapis.com/fcm/send/...\",\n      keys: %{\n        p256dh: \"client_public_key\",\n        auth: \"client_auth_secret\"\n      },\n      metadata: \"some_internal_id_or_map\"\n    }\n\n    ExNudge.send_notification(subscription, \"Hello, World!\")","ref":"ExNudge.html#module-quick-start"},{"type":"function","title":"ExNudge.generate_vapid_keys/0","doc":"Generates a new VAPID key pair for your application.","ref":"ExNudge.html#generate_vapid_keys/0"},{"type":"function","title":"Examples - ExNudge.generate_vapid_keys/0","doc":"iex> ExNudge.generate_vapid_keys()\n    %{\n      public_key: \"BK8nBpIE2tsGVt8...\",\n      private_key: \"aBcDeFgHiJkLmNoPqRsTuVwXyZ...\"\n    }","ref":"ExNudge.html#generate_vapid_keys/0-examples"},{"type":"type","title":"ExNudge.known_error/0","doc":"","ref":"ExNudge.html#t:known_error/0"},{"type":"function","title":"ExNudge.send_notification/3","doc":"Sends a web push notification to a single subscription.","ref":"ExNudge.html#send_notification/3"},{"type":"function","title":"Options - ExNudge.send_notification/3","doc":"- `:ttl` - Time to live in seconds (default: 60)\n  - `:urgency` - Message urgency level (default: :normal)\n  - `:topic` - Topic for message replacement","ref":"ExNudge.html#send_notification/3-options"},{"type":"function","title":"Examples - ExNudge.send_notification/3","doc":"iex> subscription = %ExNudge.Subscription{...}\n    iex> ExNudge.send_notification(subscription, \"Hello!\")\n    {:ok, %HTTPoison.Response{status_code: 201}}\n\n    iex> ExNudge.send_notification(subscription, \"Urgent!\", urgency: :high, ttl: 300)\n    {:ok, %HTTPoison.Response{status_code: 201}}","ref":"ExNudge.html#send_notification/3-examples"},{"type":"function","title":"ExNudge.send_notifications/3","doc":"Sends web push notifications to multiple subscriptions concurrently.\n\nReturns a list of results in the same order as the input subscriptions.","ref":"ExNudge.html#send_notifications/3"},{"type":"function","title":"Examples - ExNudge.send_notifications/3","doc":"iex> subscriptions = [sub1, sub2, sub3, sub4]\n    iex> ExNudge.send_notifications(subscriptions, \"Broadcast message\")\n    [\n      {:ok, %ExNudge.Subscription{}, %HTTPoison.Response{}},\n      {:error, :subscription_expired},\n      {:error, :invalid_subscription},\n      {:error, %ExNudge.Subscription{}, %HTTPoison.Response{}},\n      {:ok, %ExNudge.Subscription{}, %HTTPoison.Response{}}\n    ]","ref":"ExNudge.html#send_notifications/3-examples"},{"type":"type","title":"ExNudge.send_options/0","doc":"","ref":"ExNudge.html#t:send_options/0"},{"type":"type","title":"ExNudge.send_result/0","doc":"","ref":"ExNudge.html#t:send_result/0"},{"type":"module","title":"ExNudge.Subscription","doc":"Subscription from a client.","ref":"ExNudge.Subscription.html"},{"type":"function","title":"ExNudge.Subscription.from_map/2","doc":"Creates a new subscription from a JSON.\nYou can use metadata to pass around some form of identification of the subscription.\nMetadata is NOT sent with the push notification request.","ref":"ExNudge.Subscription.html#from_map/2"},{"type":"function","title":"Examples - ExNudge.Subscription.from_map/2","doc":"iex> data = %{\n    ...>   \"endpoint\" => \"https://fcm.googleapis.com/...\",\n    ...>   \"keys\" => %{\n    ...>     \"p256dh\" => \"client_public_key\",\n    ...>     \"auth\" => \"client_auth_secret\"\n    ...>   }\n    ...> }\n    iex> ExNudge.Subscription.from_map(data)\n    {:ok, %ExNudge.Subscription{...}}\n\n    iex> data = %{\n    ...>   \"endpoint\" => \"https://fcm.googleapis.com/...\",\n    ...>   \"keys\" => %{\n    ...>     \"p256dh\" => \"client_public_key\",\n    ...>     \"auth\" => \"client_auth_secret\"\n    ...>   }\n    ...> }\n    iex> metadata = \"subscription_id_1\"\n    iex> ExNudge.Subscription.from_map(data, metadata)\n    {:ok, %ExNudge.Subscription{...}}","ref":"ExNudge.Subscription.html#from_map/2-examples"},{"type":"type","title":"ExNudge.Subscription.t/0","doc":"","ref":"ExNudge.Subscription.html#t:t/0"},{"type":"function","title":"ExNudge.Subscription.valid?/1","doc":"Validates that a subscription has all required fields and valid format.","ref":"ExNudge.Subscription.html#valid?/1"},{"type":"module","title":"ExNudge.Encryption","doc":"Handles payload encryption according to RFC 8291 and RFC 8292.","ref":"ExNudge.Encryption.html"},{"type":"function","title":"ExNudge.Encryption.encrypt/2","doc":"","ref":"ExNudge.Encryption.html#encrypt/2"},{"type":"type","title":"ExNudge.Encryption.encrypted_payload/0","doc":"","ref":"ExNudge.Encryption.html#t:encrypted_payload/0"},{"type":"function","title":"ExNudge.Encryption.generate_keypair/0","doc":"","ref":"ExNudge.Encryption.html#generate_keypair/0"},{"type":"function","title":"ExNudge.Encryption.generate_salt/0","doc":"","ref":"ExNudge.Encryption.html#generate_salt/0"},{"type":"module","title":"ExNudge.VAPID","doc":"Handles VAPID (Voluntary Application Server Identification) authentication.","ref":"ExNudge.VAPID.html"},{"type":"function","title":"ExNudge.VAPID.generate_vapid_keys/0","doc":"","ref":"ExNudge.VAPID.html#generate_vapid_keys/0"},{"type":"function","title":"ExNudge.VAPID.get_keys/0","doc":"","ref":"ExNudge.VAPID.html#get_keys/0"},{"type":"function","title":"ExNudge.VAPID.sign_jwt/2","doc":"","ref":"ExNudge.VAPID.html#sign_jwt/2"},{"type":"type","title":"ExNudge.VAPID.vapid_keys/0","doc":"","ref":"ExNudge.VAPID.html#t:vapid_keys/0"},{"type":"module","title":"ExNudge.Utils","doc":"Utility functions for the ExNudge library.","ref":"ExNudge.Utils.html"},{"type":"function","title":"ExNudge.Utils.safe_url_decode/1","doc":"","ref":"ExNudge.Utils.html#safe_url_decode/1"},{"type":"function","title":"ExNudge.Utils.url_encode/1","doc":"","ref":"ExNudge.Utils.html#url_encode/1"},{"type":"extras","title":"ExNudge","doc":"# ExNudge\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_nudge.svg)](https://hex.pm/packages/ex_nudge)\n[![Documentation](https://img.shields.io/badge/docs-hexdocs-blue.svg)](https://hexdocs.pm/ex_nudge)\n[![CI](https://github.com/Joraeuw/ex_nudge/workflows/CI/badge.svg)](https://github.com/Joraeuw/ex_nudge/actions)\n[![Coverage](https://coveralls.io/repos/github/Joraeuw/ex_nudge/badge.svg)](https://coveralls.io/github/Joraeuw/ex_nudge)\n\nExNudge is a pure elixir library that allows easy multi-platform integration with web push notifications and encryption of messages described under [RFC 8291](https://www.rfc-editor.org/rfc/rfc8291.html) and [RFC 8292](https://www.rfc-editor.org/rfc/rfc8292.html)","ref":"readme.html"},{"type":"extras","title":"Features - ExNudge","doc":"- **RFC 8291 Compliant** - Full Web Push Protocol implementation\n- **VAPID Support/RFC 8292 Compliant** - Voluntary Application Server Identification  \n- **AES-GCM Encryption** - Secure payload encryption\n- **Concurrent Sending** - Send to multiple subscriptions simultaneously\n- **Telemetry integration** - Telemetry on sent notifications","ref":"readme.html#features"},{"type":"extras","title":"Installation - ExNudge","doc":"Add `ex_nudge` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_nudge, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - ExNudge","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"1. Generate VAPID Keys - ExNudge","doc":"```elixir\nkeys = ExNudge.generate_vapid_keys()\nIO.puts(\"Public Key: #{keys.public_key}\")\nIO.puts(\"Private Key: #{keys.private_key}\")\n```","ref":"readme.html#1-generate-vapid-keys"},{"type":"extras","title":"2. Configure Your Application - ExNudge","doc":"```elixir\n# config/config.exs\nconfig :ex_nudge,\n  vapid_subject: \"mailto:your-email@example.com\",\n  vapid_public_key: \"your_public_key_here\",\n  vapid_private_key: \"your_private_key_here\"\n```","ref":"readme.html#2-configure-your-application"},{"type":"extras","title":"3. Send a Notification - ExNudge","doc":"```elixir\n# Create a subscription and store it somewhere (typically in your database)\nsubscription = %ExNudge.Subscription{\n  endpoint: \"https://fcm.googleapis.com/fcm/send/...\",\n  keys: %{\n    p256dh: \"client_public_key\",\n    auth: \"client_auth_secret\"\n  },\n  metadata: \"any metadata\"\n}\n\n# Send the notification\ncase ExNudge.send_notification(subscription, \"Hello, World!\") do\n  {:ok, response} -> \n    IO.puts(\"Notification sent successfully!\")\n  {:error, :subscription_expired} -> \n    IO.puts(\"Subscription has expired, remove from database\")\n  {:error, reason} -> \n    IO.puts(\"Failed to send: #{inspect(reason)}\")\nend\n```","ref":"readme.html#3-send-a-notification"},{"type":"extras","title":"Batch Sending - ExNudge","doc":"```elixir\nsubscriptions = [subscription1, subscription2, subscription3]\nresults = ExNudge.send_notifications(subscriptions, \"Multicast message\")\n\n# Process results\nresults\n|> Enum.each(fn \n  {:ok, subscription, _response} -> \n    IO.puts(\"Successfully sent notification for #{subscription.metadata}\")\n    \n  {:error, subscription, :subscription_expired} -> \n    # Remove expired subscription from database\n    MyApp.remove_subscription(subscription)\n    IO.puts(\"Removed expired subscription: #{subscription.metadata}\")\n    \n  {:error, subscription, %HTTPoison.Response{status_code: status_code}} -> \n    IO.puts(\"HTTP error #{status_code} for #{subscription.metadata}\")\n    \n  {:error, subscription, reason} -> \n    IO.puts(\"Failed to send to #{subscription.metadata}: #{inspect(reason)}\")\nend)\n\n# You can also configure concurrency which defaults to System.schedulers_online() * 2\nresults = ExNudge.send_notifications(\n  subscriptions, \n  \"Multicast message\", \n  concurrency: 10\n)\n```","ref":"readme.html#batch-sending"},{"type":"extras","title":"Custom Options - ExNudge","doc":"```elixir\n# Send with custom options\nExNudge.send_notification(subscription, message, [\n  ttl: 3600,                    # Time to live (seconds)\n  urgency: :high,               # :very_low, :low, :normal, :high\n  topic: \"breaking_news\"        # Replace previous messages with same topic\n])\n```","ref":"readme.html#custom-options"},{"type":"extras","title":"Telemetry - ExNudge","doc":"```elixir\n# Attach telemetry handler\n:telemetry.attach(\"my-handler\", [:ex_nudge, :send_notification], fn name, measurements, metadata, config ->\n  case metadata.status do\n    :success ->\n      Logger.info(\"Notification sent successfully\",\n        duration: measurements.duration,\n        endpoint: metadata.endpoint)\n    :error ->\n      Logger.error(\"Notification failed\",\n        error: metadata.error_reason,\n        http_status: metadata.http_status_code)\n  end\nend, nil)\n```","ref":"readme.html#telemetry"},{"type":"extras","title":"Browser Integration - ExNudge","doc":"","ref":"readme.html#browser-integration"},{"type":"extras","title":"JavaScript Client Code - ExNudge","doc":"```javascript\nnavigator.serviceWorker.register('/sw.js');\n\nconst permission = await Notification.requestPermission();\nif (permission === 'granted') {\n  const registration = await navigator.serviceWorker.ready;\n  const subscription = await registration.pushManager.subscribe({\n    userVisibleOnly: true,\n    applicationServerKey: 'your_vapid_public_key'\n  });\n  \n  // Send subscription to your server\n  await fetch('/api/subscriptions', {\n    method: 'POST',\n    headers: { 'Content-Type': 'application/json' },\n    body: JSON.stringify(subscription)\n  });\n}\n```","ref":"readme.html#javascript-client-code"},{"type":"extras","title":"Service Worker (sw.js) - ExNudge","doc":"Keep in mind that sw.js, icon-192x192.png and badge-72x72.png usually are statically served files.  \nTake a look into the [ServiceWorkerRegistration documentation](https://developer.mozilla.org/en-US/docs/Web/API/ServiceWorkerRegistration/showNotification#options) for more details on options.\n\n```javascript\nself.addEventListener('push', event => {\n  const data = event.data ? event.data.text() : 'Default message';\n  \n  const options = {\n    body: data,\n    icon: '/icon-192x192.png',\n    badge: '/badge-72x72.png',\n    vibrate: [100, 50, 100],\n    data: { url: \"/\" }\n  };\n  \n  event.waitUntil(\n    self.registration.showNotification('App Name', options)\n  );\n});\n\nself.addEventListener('notificationclick', event => {\n  event.notification.close();\n  event.waitUntil(clients.openWindow(event.notification.data.url));\n});\n```","ref":"readme.html#service-worker-sw-js"},{"type":"extras","title":"Contributing - ExNudge","doc":"1. Fork the repository\n2. Create a feature branch\n3. Write tests for your changes\n4. Ensure all tests pass: `mix test`\n5. Submit a pull request","ref":"readme.html#contributing"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0]","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release\n- RFC 8291 compliant Web Push Protocol implementation\n- VAPID (RFC 8292) authentication support\n- AES-GCM payload encryption\n- Error handling\n- Telemetry integration\n- Concurrent batch sending support\n- Complete test suite\n- Documentation and examples","ref":"changelog.html#added"},{"type":"extras","title":"Features - Changelog","doc":"- Send individual notifications with `ExNudge.send_notification/3`\n- Send batch notifications with `ExNudge.send_notifications/3`\n- Generate VAPID keys with `ExNudge.generate_vapid_keys/0`\n- Support for TTL, urgency, and topic headers\n- Automatic subscription validation\n- Detailed error reporting","ref":"changelog.html#features"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}